/*
 * main.c
 *
 *  Created on: Nov 27, 2018
 *      Author: Rosh
 *
 *  Notes:
 *  - FRAM read/write speed/energy testing
 */

#include "conf_EPD.h"

#include <stdint.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "driverlib.h"


// general utilities
#include "utils/myuart.h"
//#include "utils/stopwatch.h"



/*******************************************************
 * Globals
 *******************************************************/
#define CLK_SPEED_8MHz      1
#define CLK_SPEED_16MHz     0

#define BUFF_SIZE 16*96

// buffers in SRAM
unsigned char SRAM_Buff1[BUFF_SIZE];
unsigned char SRAM_Buff2[BUFF_SIZE];

// buffers in FRAM

#pragma PERSISTENT(FRAM_Buff1)
unsigned char FRAM_Buff1[BUFF_SIZE] = {0};
#pragma PERSISTENT(FRAM_Buff2)
unsigned char FRAM_Buff2[BUFF_SIZE] = {0};




//unsigned int FreqLevel = 7;
//int uartsetup=0;

/*******************************************************
 * FUNC DEFS
 *******************************************************/
// benchmarks
void benchmark_exp0(void);
void _buffer_populate(uint8_t *Buff_ptr, uint8_t data_byte);

// setup related
//void uart_setup(void);
//void clock_setup(void);

// helpers
void _delay(uint32_t d);

// debug
void setupDebugPins(void);

/*************************************************************************************
 * SETUP
 *************************************************************************************/
/*
void uart_setup(void){
    uartsetup=0;
    uartinit();
}

void clock_setup(void){

#if CLK_SPEED_8MHz
    //Set DCO Frequency to 8MHz
    CS_setDCOFreq(CS_DCORSEL_0, CS_DCOFSEL_6);

    //configure MCLK, SMCLK to be source by DCOCLK
    CS_initClockSignal(CS_MCLK,  CS_DCOCLK_SELECT,  CS_CLOCK_DIVIDER_1); //16mhz
    CS_initClockSignal(CS_SMCLK, CS_DCOCLK_SELECT,  CS_CLOCK_DIVIDER_1); // 16mhz

#endif

#if CLK_SPEED_16MHz

#endif

    __bis_SR_register(GIE);

    //Verify if the Clock settings are as expected
    volatile uint32_t clockValue;
    clockValue = CS_getMCLK();
    clockValue = CS_getACLK();
    clockValue = CS_getSMCLK();
    if(clockValue);
}

*/
/*************************************************************************************
 * buffer related
 *************************************************************************************/
void _buffer_populate(uint8_t *Buff_ptr, uint8_t data_byte){
    uint32_t i;
    for(i=0; i < BUFF_SIZE; i++){
        Buff_ptr[i] = data_byte;
    }
}


/*************************************************************************************
 * DEBUG
 *************************************************************************************/

void setupDebugPins(void){
    /* launchpad LEDs */
    GPIO_setAsOutputPin( GPIO_PORT_P1, GPIO_PIN0 );
    GPIO_setAsOutputPin( GPIO_PORT_P1, GPIO_PIN1 );
    GPIO_setOutputLowOnPin( GPIO_PORT_P1, GPIO_PIN0 );
    GPIO_setOutputLowOnPin( GPIO_PORT_P1, GPIO_PIN1 );
}

/*************************************************************************************
 * MAIN
 *************************************************************************************/
void main(void)
{
    /* mandatory init stuff */
    WDTCTL = WDTPW | WDTHOLD;     //Stop WDT
    PM5CTL0 &= ~LOCKLPM5; // Disable the GPIO power-on default high-impedance mode to activate previously configured port settings

    uint16_t i;

    clock_setup();
    uart_setup();
    setupDebugPins();



    _DBGUART("\r\n -- FINISHED SYS/BOARD SETUP 2-- \r\n");
    _DBGUART("SMLK= %l ; MCLK= %l ; ACLK=%l \r\n", CS_getSMCLK(), CS_getMCLK(), CS_getACLK());


    _delay(1000000);

    /* initialize the buffers in SRAM */
    _buffer_populate(SRAM_Buff1, (uint8_t)0xFF); // buff init
    _buffer_populate(SRAM_Buff2, (uint8_t)0xFF); // buff init

    _delay(1000000);

    /* fram write */
    for(i=0; i<1000; i++){
        memcpy(FRAM_Buff1, SRAM_Buff1, BUFF_SIZE);
        memcpy(FRAM_Buff2, SRAM_Buff2, BUFF_SIZE);
    }

    _delay(1000000);

    /* fram read */
    for(i=0; i<1000; i++){
        memcpy(SRAM_Buff1, FRAM_Buff1, BUFF_SIZE);
        memcpy(SRAM_Buff2, FRAM_Buff2, BUFF_SIZE);
    }

    _delay(1000000);


    _DBGUART("\r\n -- DONE -- \r\n");
}



/*************************************************************************************
 * BENCHMARKING
 *************************************************************************************/





/*************************************************************************************
 * HELPER FUNCTIONS
 *************************************************************************************/
void _delay(uint32_t d){
    uint32_t i;
    for (i=0;i<d;i++){__no_operation();}
}




void UARTIntHandler()
{}
